/*
 * system.c
 *
 *  Created on: 26 sty 2017
 *      Author: dsosnowski
 */



#include "chip.h"
#include "system.h"

//--------------------------------------------------------------------------
void SystemInit(void){
	/* IRC should be powered up */
	Chip_SYSCTL_PowerUp(SYSCTL_POWERDOWN_IRC_PD);
	Chip_SYSCTL_PowerUp(SYSCTL_POWERDOWN_IRCOUT_PD);
	Chip_SYSCTL_PowerUp(SYSCTL_SLPWAKE_SYSOSC_PD);

	/* Set system PLL input to main oscillator */
	Chip_Clock_SetSystemPLLSource(SYSCTL_PLLCLKSRC_MAINOSC);

	/* Power down PLL to change the PLL divider ratio */
	Chip_SYSCTL_PowerDown(SYSCTL_POWERDOWN_SYSPLL_PD);

	/* Setup PLL for main oscillator rate (FCLKIN = 12MHz) * 4 = 48MHz
	   MSEL = 3 (this is pre-decremented), PSEL = 1 (for P = 2)
	   FCLKOUT = FCLKIN * (MSEL + 1) = 12MHz * 4 = 48MHz
	   FCCO = FCLKOUT * 2 * P = 48MHz * 2 * 2 = 192MHz (within FCCO range) */
	Chip_Clock_SetupSystemPLL(3, 1);

	/* Powerup system PLL */
	Chip_SYSCTL_PowerUp(SYSCTL_POWERDOWN_SYSPLL_PD);

	/* Wait for PLL to lock */
	while (!Chip_Clock_IsSystemPLLLocked()) {}

	/* Set system clock divider to 1 */
	Chip_Clock_SetSysClockDiv(1);

	/* Setup FLASH access to 3 clocks */
	Chip_FMC_SetFLASHAccess(FLASHTIM_50MHZ_CPU);

	/* Set main clock source to the system PLL. This will drive 48MHz
	   for the main clock and 48MHz for the system clock */
	Chip_Clock_SetMainClockSource(SYSCTL_MAINCLKSRC_PLLOUT);

	/* Enable IOCON clock */
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
   // Chip_GPIO_Init(LPC_GPIO);
    Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_GPIO);

    SystemIOCONInit();
}

//--------------------------------------------------------------------------
void SystemIOCONInit(void){
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO0_10, IOCON_FUNC1 | IOCON_MODE_PULLUP);	//SWC dis
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO0_11, IOCON_FUNC1 | IOCON_MODE_PULLUP);	//SWD dis
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO1_0, IOCON_FUNC1 | IOCON_MODE_PULLUP | IOCON_DIGMODE_EN);		//switch to normal port oper
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO1_1, IOCON_FUNC1 | IOCON_MODE_PULLUP | IOCON_DIGMODE_EN);		//switch to normal port oper
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO1_2, IOCON_FUNC1 | IOCON_MODE_PULLUP | IOCON_DIGMODE_EN);		//switch to normal port oper
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO1_3, IOCON_FUNC1 | IOCON_MODE_PULLUP | IOCON_DIGMODE_EN);		//switch to normal port oper
}

//---------------------------------------------------
void IoMaskedValueSet(LPC_GPIO_T *pGPIO, uint8_t port, uint32_t mask, uint32_t val){
	//LPC_GPIO[port].DATA[mask] = val;
	pGPIO[port].DATA[mask] = val;
}

//--------------------------------------------------------------------------
void SystemSysTickInit(uint16_t period){
	SysTick->LOAD = (((MY_MAIN_CLOCK / 2) / (1000 / period)) - 1);
	SysTick->VAL = 0x00;
	SysTick->CTRL = (1<<SysTick_CTRL_ENABLE_Pos | 1<<SysTick_CTRL_CLKSOURCE_Pos);	//start timer, disable int, core clk
}

//--------------------------------------------------------------------------
uint32_t SystemSysTickCheck(void){
	if (((SysTick->CTRL) & (1<<SysTick_CTRL_COUNTFLAG_Pos)) > 0){
		return 1;
	}
	return 0;
}
