/*
 * mydraw.c
 *
 *  Created on: 06.02.2018
 *      Author: Procesorowiec
 */


#include "chip.h"
#include "system.h"
#include "mydelay.h"
#include "myspi.h"
#include "drvST7735.h"
#include "mydraw.h"


struct {
	uint16_t activeColor;
	uint16_t backgroundColor;
	uint16_t screenWidth;
	uint16_t screenHeight;
} draw;

//================================================================
static void MyDrawScreenSizeSet(void){
	draw.screenWidth = (uint16_t)DrvWidthGet();
	draw.screenHeight = (uint16_t)DrvHeightGet();
}

//----------------------------------------------------------------
static void MyDrawPixelFill(uint16_t *colorSrc){
	DrvPixelFill(colorSrc);
}

static void MyDrawActiveAreaSet(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2){
	DrvWindowSet((uint8_t)x1, (uint8_t)y1, (uint8_t)x2, (uint8_t)y2);
}

//================================================================
void MyDrawInit(void){
	MyDrawScreenSizeSet();

	MyDrawColorSet(DRAW_WHITE);
	MyDrawScreenFill(DRAW_BLACK);
}

//----------------------------------------------------------------
//uint16_t MyDrawDefinedColorGet(uint8_t colorIndex) {
//	return draw.defColors[colorIndex];
//}

//----------------------------------------------------------------
void MyDrawColorSet(uint16_t color) {
	draw.activeColor = color;
	draw.backgroundColor = DRAW_DEFAUL_BACKCOLOR;
}

//----------------------------------------------------------------
uint16_t *MyDrawColorGet(uint8_t colorIndex) {
	if (colorIndex > 0) return &draw.activeColor;
	return &draw.backgroundColor;
}

//----------------------------------------------------------------
void MyDrawFill(uint8_t active) {
	if (active > 0){
		MyDrawPixelFill(&draw.activeColor);
	} else {
		MyDrawPixelFill(&draw.backgroundColor);
	}
}

//----------------------------------------------------------------
uint16_t MyDrawScreenWidthGet(void) {
	return draw.screenWidth;
}

//----------------------------------------------------------------
uint16_t MyDrawScreenHeightGet(void) {
	return draw.screenHeight;
}

//----------------------------------------------------------------
uint16_t MyDrawCoordSet(int16_t x1, int16_t y1, int16_t x2, int16_t y2) {
	if (x1 >= draw.screenWidth) return 0;
	if (y1 >= draw.screenHeight) return 0;
	if (x2 >= draw.screenWidth) x2 = (draw.screenWidth - 1);
	if (y2 >= draw.screenHeight) y2 = (draw.screenHeight - 1);
	MyDrawActiveAreaSet(x1, y1, x2, y2);
 	return 1;
}

//----------------------------------------------------------------
void MyDrawCoordFill(int16_t x1, int16_t y1, int16_t x2, int16_t y2, uint16_t color) {
	uint16_t i;

	if  (MyDrawCoordSet(x1,y1,x2,y2) > 0){
		MyDrawColorSet(color);

		for(; y1 <= y2; y1++) {
			for(i = x1; i <= x2; i++) {
				MyDrawFill(1);
			}
		}
    }
}

//----------------------------------------------------------------
void MyDrawScreenFill(uint16_t color) {
	MyDrawCoordFill(0, 0, (draw.screenWidth - 1), (draw.screenHeight -1), color);
}

//----------------------------------------------------------------
void MyDrawHLine(int16_t x1, int16_t x2, int16_t y, int16_t width, uint16_t color) {
	if (width > 0){
		MyDrawCoordFill(x1,y,x2,(y + width - 1),color);
	}
}

//----------------------------------------------------------------
void MyDrawVLine(int16_t y1, int16_t y2, int16_t x, int16_t width, uint16_t color) {
	if (width > 0){
		MyDrawCoordFill(x,y1,(x + width - 1),y2,color);
	}
}

