/*
 * myfonts.c
 *
 *  Created on: 7 lut 2018
 *      Author: dsosnowski
 */

 
 
#include "chip.h"
#include "mydraw.h"
#include "fontTypeDefs.h"
#include "myfonts.h"

//=========================================================
static uint16_t FontsScreenWidthGet(void){
	return MyDrawScreenWidthGet();
}

//---------------------------------------------------------
static uint16_t FontsScreenHeightGet(void){
	return MyDrawScreenHeightGet();
}

//---------------------------------------------------------
static void FontsColorSet(uint16_t color){
	MyDrawColorSet(color);
}

//---------------------------------------------------------
static void FontsFill(uint8_t active){
	MyDrawFill(active);
}

//---------------------------------------------------------
uint16_t FontsActiveAreaSet(int16_t x1, int16_t y1, int16_t x2, int16_t y2) {
	return MyDrawCoordSet(x1,y1,x2,y2);
}

//=========================================================
struct {
	uint16_t x1;
	uint16_t x2;
} fontLine;


//---------------------------------------------------------
void FontsInit(void){
	fontLine.x1 = 0;
	fontLine.x2 = (FontsScreenWidthGet() - 1);
}

//---------------------------------------------------------
uint16_t FontsStrPixelLen(uint8_t *src, FONT_TYPE_T font){
	uint8_t fontChar;
	uint16_t pixelsLen = 0;
	
	while ((*src > 0) && (src > 0)){
		if (*src != ' '){
			fontChar = ((*src)- font.firstCharCode);
			pixelsLen += font.charInfo[fontChar].width;		
		} else {
			pixelsLen += font.spaceWidth;
		}
		pixelsLen += font.interspace;
		src++;
	}

	if (pixelsLen > 0) pixelsLen -= font.interspace;
	return pixelsLen;
}

//-------------------------------------------------------------------
void FontsVal2String(int32_t val, uint16_t dotPos, uint8_t *dst){
	uint8_t result[16];
	int32_t i;

	for (i = 0; i < 15; i++){
		if ((dotPos > 0) && (dotPos == i)){
			result[i] = ',';
			i++;
			if (val == 0) result[i] = '0';
		} else {
			result[i] = (val % 10) + '0';
			val /= 10;			
		}
		
		if (dotPos == 0){
			if (val == 0) break;			
		} else {
			if ((i >= dotPos) && (val == 0)) break;
		}
	}
	
	for (; i >= 0; i--){
		*dst = result[i];
		dst++;
	}	
	*dst = 0;
}

//---------------------------------------------------------
void FontsStrLineRangeSet(uint16_t x1, uint16_t x2){
	if (x1 >= FontsScreenWidthGet()) x1 = (FontsScreenWidthGet() - 1);
	if (x2 >= FontsScreenWidthGet()) x2 = (FontsScreenWidthGet() - 1);
	fontLine.x1 = x1;
	fontLine.x2 = x2;	
}

//---------------------------------------------------------
void FontsStrWrite(uint8_t *src, uint16_t xStart, uint16_t yStart, FONT_TYPE_T font, uint16_t color){	
	uint16_t y;
	uint16_t x;
	
	uint8_t *charPt;
	uint8_t fontCharIndex;
	uint16_t fontBitmapOffset;
	uint8_t *fontBitmapSrc;
	uint8_t fontWidth;
	uint8_t hBytes;
	uint8_t hBitMask;	
	
	FontsColorSet(color);
	
	if (FontsActiveAreaSet(fontLine.x1, yStart, fontLine.x2, (yStart + font.height - 1 + font.interspace)) > 0){

		for(y = 0; y < font.height; y++){
			
			hBytes = (y/8);
			hBitMask = (1 << (y % 8));
			charPt = src;
			
			x = fontLine.x1;
			while (x < xStart){
				FontsFill(0);
				x++;
			}
			
			while ((*charPt > 0) && (charPt > 0)){
				if (*charPt != ' ') {
					fontCharIndex = ((*charPt) - font.firstCharCode);

					fontBitmapOffset = font.charInfo[fontCharIndex].offset;
					fontBitmapOffset += (hBytes * (font.charInfo[fontCharIndex].width));

					fontBitmapSrc = font.bitmap;
					fontBitmapSrc += fontBitmapOffset;

					for (fontWidth = 0; fontWidth < font.charInfo[fontCharIndex].width; fontWidth++){
						FontsFill(((*fontBitmapSrc) & hBitMask));
						fontBitmapSrc++;
						x++;
					}
				} else {
					for (fontWidth = 0; fontWidth < font.spaceWidth; fontWidth++){
						FontsFill(0);
						x++;
					}										
				}

				charPt++;
				if ((*charPt > 0) && (charPt > 0)){
					for (fontWidth = 0; fontWidth < font.interspace; fontWidth++){
						FontsFill(0);
						x++;
					}
				}
			}
			


			while (x <= fontLine.x2){
				FontsFill(0);
				x++;
			}
		}

		/*for(y = 0; y < 1; y++){
			for(x = fontLine.x1; x <= fontLine.x2; x++){
				FontsFill(0);
			}
		}*/
	}
}

//---------------------------------------------------------
void FontsStrWriteReverseXPos(uint8_t *src, uint16_t xEnd, uint16_t yStart, FONT_TYPE_T font, uint16_t color){	
	FontsStrWrite(src, (xEnd - FontsStrPixelLen(src, font)), yStart, font, color);
}

//---------------------------------------------------------
void FontsValueWrite(int32_t val, uint16_t xEnd, uint16_t yStart, FONT_TYPE_T font, uint16_t color){	

	uint8_t str[16];
	
	FontsVal2String(val, 0, str);
	
	FontsStrWriteReverseXPos(str, xEnd, yStart, font, color);
}

//---------------------------------------------------------
void FontsValueWriteWdot(int32_t val, uint16_t dot, uint16_t xEnd, uint16_t yStart, FONT_TYPE_T font, uint16_t color){

	uint8_t str[16];

	FontsVal2String(val, dot, str);

	FontsStrWriteReverseXPos(str, xEnd, yStart, font, color);
}
