/*
 * myspi.c
 *
 *  Created on: 5 lut 2018
 *      Author: dsosnowski
 */

#include "chip.h"
#include "system.h"
#include "myspi.h"

#define	MY_SPI			LPC_SSP0
#define MYSPI_FREQ		4000000

#define SPI_PORT		0
#define SPICS_PIN		2
#define SPICLK_PIN		6
#define SPIDTA_PIN		9
#define SPIDC_PIN		7
#define SPIRST_PIN		10
#define SPIBKL_PIN		8
#define SPIPIN_MASK		(1<<SPICS_PIN | 1<<SPICLK_PIN | 1<<SPIDTA_PIN | 1<<SPIDC_PIN | 1<<SPIRST_PIN | 1<<SPIBKL_PIN)


//-------------------------------------------------------
void MySpiInit(void){
	//spi port
	IoMaskedValueSet(LPC_GPIO, SPI_PORT, SPIPIN_MASK, SPIPIN_MASK);
	Chip_GPIO_SetPortDIROutput(LPC_GPIO, SPI_PORT, SPIPIN_MASK);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SSP0);
	Chip_Clock_SetSSP0ClockDiv(1);
	Chip_SYSCTL_PeriphReset(RESET_SSP0);

	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO0_6, IOCON_FUNC2);		//SCK0
	Chip_IOCON_PinLocSel(LPC_IOCON, IOCON_SCKLOC_PIO0_6);			//SCK0
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO0_9, IOCON_FUNC1);		//MOSI0
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO0_2, IOCON_FUNC1);		//SSEL0
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO0_7, IOCON_FUNC0);		//DC - no hard control
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO0_10, IOCON_FUNC1);	//RST - no hard control
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO0_8, IOCON_FUNC0);		//RST - no hard control

	Chip_SSP_SetFormat(MY_SPI, SSP_BITS_8, SSP_FRAMEFORMAT_SPI, SSP_CLOCK_CPHA1_CPOL1);
	Chip_SSP_SetClockRate(MY_SPI, ((MY_MAIN_CLOCK/2/MYSPI_FREQ) - 1), 2);
	Chip_SSP_Set_Mode(MY_SPI, SSP_MODE_MASTER);
	//Chip_SSP_EnableLoopBack(SPI_PORT);

	Chip_SSP_Enable(MY_SPI);

	/* Clear all remaining frames in RX FIFO */
	while (MySpiReceiveReady()) {
		MySpiReceiveByte();
	}
}

//-----------------------------------------------------
void MySpiSendByte(uint8_t tx_data) {
	MY_SPI->DR = tx_data;
}

//-----------------------------------------------------
uint8_t MySpiReceiveByte(void){
	return (uint8_t)MY_SPI->DR;
}

uint8_t MySpiGetStatus(SSP_STATUS_T Stat){
	return (MY_SPI->SR & Stat) ? 1 : 0;
}

//-----------------------------------------------------
uint8_t MySpiReadyToSend(void){
	return MySpiGetStatus(SSP_STAT_TNF);
}

//-----------------------------------------------------
uint8_t MySpiSendComplete(void){
	return MySpiGetStatus(SSP_STAT_BSY);
}

//-----------------------------------------------------
uint8_t MySpiReceiveReady(void){
	return MySpiGetStatus(SSP_STAT_RNE);
}

//-----------------------------------------------------
void MySpiDataModeSet(void) {
	IoMaskedValueSet(LPC_GPIO, SPI_PORT, 1<<SPIDC_PIN, 1<<SPIDC_PIN);
}

//-----------------------------------------------------
void MySpiCommandModeSet(void) {
	IoMaskedValueSet(LPC_GPIO, SPI_PORT, 1<<SPIDC_PIN, 0);
}

//-----------------------------------------------------
void MySpiHResetSet(void) {
	IoMaskedValueSet(LPC_GPIO, SPI_PORT, 1<<SPIRST_PIN, 0);
}

//-----------------------------------------------------
void MySpiHResetRelase(void) {
	IoMaskedValueSet(LPC_GPIO, SPI_PORT, 1<<SPIRST_PIN, 1<<SPIRST_PIN);
}
