/*
===============================================================================
 Name        : genTFT.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include <cr_section_macros.h>

#include "system.h"
#include "mydelay.h"
#include "cortexino.h"
#include "serial.h"
#include "serialDebug.h"

#include "buttons.h"
#include "encoder.h"

#include "myspi.h"
#include "drvST7735.h"
#include "mydraw.h"
#include "myfonts.h"

#include "mainscreen.h"
#include "menu.h"
#include "settings.h"
#include "gen.h"




//-----------------------------------------------------------------------------
int main(void) {

	uint8_t i;
	uint8_t state;
	uint8_t refresh = 1;

	SystemInit();
	MyDelayInit();
	CortexinoInit();
	SerialInit(19200);

	MySpiInit();
	ST7735Init(3);
	MyDrawInit();
	FontsInit();

	ButtonsInit();
	EncoderInit();

	MianScreenFrame();
	MenuInit();
	SettInit();
	GenInit();

	SystemSysTickInit(MAIN_DELAY);

	while(1){
		while (SystemSysTickCheck() == 0){};


		//-----------------------------------------------------------------
		ButtonsProcess();
		if (ButtonsClickCheck() > 0){
			if (ButtonsClickGet() == BUTTONS_ACTION_ENTER){
				MenuEditLevelChange(1);
				refresh = 1;
			}
		}
		if (ButtonsActionCheck() > 0){
			if (ButtonsStateGet() & BUTTONS_ACTION_LONG){
				MenuPosChange(1);
				MenuEditLevelChange(0);
				refresh = 1;
			}
		}

		/*if (encoderPause > 0) {
			encoderPause--;
			if (EncoderActionCheck() > 0){
				EncoderValueGet();
			}
		}
		//-----------------------------------------------------------------
		if (EncoderActionCheck() > 0){
			//EncoderValueGet();
			if (encoderPause == 0){
				SettFreqChange(EncoderValueGet() * MenuEditMultiGet());
				GenSett(SettFreqGet());
				GenTimerConfogUpdate();
				encoderPause = 20;
				refresh = 1;
			}
		}*/
		EncoderProc();
		if (EncoderActionCheck() > 0){
			SettChange((EncoderValueGet() * MenuEditValueGet()) , MenuPosGet());
			GenSett(SettFreqGet(), SettDutyGet());
			GenTimerConfogUpdate();
			refresh = 1;
		}

		//-----------------------------------------------------------------
		if (refresh > 0){
			refresh = 0;

			CortexinoLed(2,LED_SET);


			MianScreenValues();
			MianScreenMarker(MenuPosGet(), MenuEditLevelGet());


			CortexinoLed(2,LED_OFF);
		}

		//-----------------------------------------------------------------
		i++;
		if (i > (400 / MAIN_DELAY)){
			i = 0;

			state++;
			if (state & 1){
				CortexinoLed(1,LED_SET);
			} else {
				CortexinoLed(1,LED_OFF);
			}
		}

		//-----------------------------------------------------------------
	}

}
