/*
 * buttons.c
 *
 *  Created on: 12.02.2018
 *      Author: Procesorowiec
 */



#include "chip.h"
#include "system.h"
#include "buttons.h"

#define BUTTONS_PORT					1
#define BUTTONS_SW1_PIN					4
//#define BUTTONS_SW2_PIN				1
//#define BUTTONS_SW3_PIN				2
//#define BUTTONS_SW4_PIN				3
#define KBD_PINS_MASK					(1<<BUTTONS_SW1_PIN)
//#define KBD_PINS_MASK					(1<<BUTTONS_SW1_PIN | 1<<BUTTONS_SW2_PIN | 1<<BUTTONS_SW3_PIN | 1<<BUTTONS_SW4_PIN)

#define BUTTONS_ACTION_MIN_CYCLE		2
#define BUTTONS_ACTION_LONG_CYCLE		(1000 / MAIN_DELAY)
#define BUTTONS_ACTION_REPEAT_CYCLE		3

#define BUTTONS_COMMON_PIN				5



struct {
	uint16_t click;
	uint16_t state;
} buttons;

//-----------------------------------------------------------
void ButtonsInit(void){

#ifdef BUTTONS_COMMON_PIN
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO1_5, IOCON_FUNC0);		//switch to normal port oper
	Chip_GPIO_SetPortDIROutput(LPC_GPIO, BUTTONS_PORT, (1<<BUTTONS_COMMON_PIN));
	Chip_GPIO_SetPinState(LPC_GPIO, BUTTONS_PORT, BUTTONS_COMMON_PIN, 0);
#endif

	Chip_GPIO_SetPortDIRInput(LPC_GPIO, BUTTONS_PORT, KBD_PINS_MASK);
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO1_4, IOCON_FUNC0 | IOCON_MODE_PULLUP | IOCON_HYS_EN | IOCON_DIGMODE_EN);
}


//------------------------------------------------------------------
static uint32_t ButtonsPortStateGet(void){
	return Chip_GPIO_GetPortValue(LPC_GPIO, BUTTONS_PORT);
}

//-----------------------------------------------------------
void ButtonsProcess(void){
	static uint16_t prev;
	static uint16_t hold;
	uint16_t pressed = 0;
	uint16_t result = 0;
	uint32_t state = 0;

	buttons.click = 0;

	state = ButtonsPortStateGet();
	//if ((state & (1<<BUTTONS_SW1_PIN)) == 0) pressed = BUTTONS_ACTION_ESC;
	//if ((state & (1<<BUTTONS_SW2_PIN)) == 0) pressed = BUTTONS_ACTION_UP;
	//if ((state & (1<<BUTTONS_SW3_PIN)) == 0) pressed = BUTTONS_ACTION_DOWN;
	//if ((state & (1<<BUTTONS_SW4_PIN)) == 0) pressed = BUTTONS_ACTION_ENTER;
	if ((state & (1<<BUTTONS_SW1_PIN)) == 0) pressed = BUTTONS_ACTION_ENTER;

	if (pressed == 0) {
		if (hold > BUTTONS_ACTION_MIN_CYCLE){
			result = prev;
			if (hold < BUTTONS_ACTION_LONG_CYCLE) {
				buttons.click = prev;
				result |= BUTTONS_ACTION_CLICK;
			} else {
				result |= BUTTONS_ACTION_RELEASE;
			}
		}

		hold = 0;

	} else {
		result = pressed;
		if (pressed != prev) hold = 0;
		if (hold < 0xFFFF) hold++;

		if (hold == BUTTONS_ACTION_LONG_CYCLE) {
			result |= BUTTONS_ACTION_LONG;
		}

		if (hold > BUTTONS_ACTION_LONG_CYCLE) {
			result |= BUTTONS_ACTION_HOLD;
		}

		if (hold > (BUTTONS_ACTION_LONG_CYCLE + BUTTONS_ACTION_REPEAT_CYCLE)){
			result |= BUTTONS_ACTION_REPEAT;
			hold -= BUTTONS_ACTION_REPEAT_CYCLE;
		}
	}

	prev = pressed;
	buttons.state = result;
}

//-----------------------------------------------------------
uint16_t ButtonsClickCheck(void){
	if (buttons.click >= 0) return 1;
	return 0;
}

//-----------------------------------------------------------
uint16_t ButtonsClickGet(void){
	return buttons.click;
}

//-----------------------------------------------------------
uint16_t ButtonsActionCheck(void){
	if (buttons.state >= BUTTONS_ACTION_LONG) return 1;
	return 0;
}

//-----------------------------------------------------------
uint16_t ButtonsStateGet(void){
	return buttons.state;
}
