/*
 * encoder.c
 *
 *  Created on: 12.02.2018
 *      Author: Procesorowiec
 */


#include "chip.h"
#include "system.h"
#include "mydelay.h"
#include "serialDebug.h"
#include "encoder.h"

#define ENCODER_PORT			1
#define ENCODER_COMMON_PIN		0
#define ENCODER_PIN1			2
#define ENCODER_PIN2			3
#define ENCODER_PIN_MASK		(1<<ENCODER_PIN1 | 1<<ENCODER_PIN2)

#define ENCODER_IDLE			0
#define ENCODER_INCREMENT		1
#define ENCODER_DECREMENT		-1
#define ENCODER_STATE_THRESHOLD	5


//------------------------------------------------------------------
struct{
	volatile int16_t pulses;
} encoder;


//------------------------------------------------------------------
void EncoderInit(void){

	#ifdef ENCODER_COMMON_PIN
		Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO1_0, IOCON_FUNC1);		//switch to normal port oper
		Chip_GPIO_SetPortDIROutput(LPC_GPIO, ENCODER_PORT, (1<<ENCODER_COMMON_PIN));
		Chip_GPIO_SetPinState(LPC_GPIO, ENCODER_PORT, ENCODER_COMMON_PIN, 0);
	#endif

	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO1_2, IOCON_FUNC1 | IOCON_MODE_PULLUP | IOCON_HYS_EN | IOCON_DIGMODE_EN);		//switch to normal port oper
	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO1_3, IOCON_FUNC1 | IOCON_MODE_PULLUP | IOCON_HYS_EN | IOCON_DIGMODE_EN);		//switch to normal port oper
	Chip_GPIO_SetPortDIRInput(LPC_GPIO, ENCODER_PORT, ENCODER_PIN_MASK);
	IoMaskedValueSet(LPC_GPIO, ENCODER_PORT, ENCODER_PIN_MASK, ENCODER_PIN_MASK);

	/*Chip_GPIO_SetPinModeEdge(LPC_GPIO, ENCODER_PORT, (1<<ENCODER_PIN1));
	Chip_GPIO_SetEdgeModeSingle(LPC_GPIO, ENCODER_PORT, (1<<ENCODER_PIN1));
	Chip_GPIO_SetModeLow(LPC_GPIO, ENCODER_PORT, (1<<ENCODER_PIN1));
	Chip_GPIO_ClearInts(LPC_GPIO, ENCODER_PORT, (1<<ENCODER_PIN1));
	Chip_GPIO_EnableInt(LPC_GPIO, ENCODER_PORT, (1<<ENCODER_PIN1));
	NVIC_EnableIRQ(EINT1_IRQn);
	*/

}

//------------------------------------------------------------------
static uint32_t EncoderPortStateGet(void){
	return Chip_GPIO_GetPortValue(LPC_GPIO, ENCODER_PORT);
}

//------------------------------------------------------------------
static int16_t EncoderDecode(void){
	uint32_t portState = EncoderPortStateGet();

	if ((portState & (1<<ENCODER_PIN1)) == 0) {
		if ((portState & (1<<ENCODER_PIN2)) == 0) {
			return ENCODER_INCREMENT;
		} else {
			return ENCODER_DECREMENT;
		}
	}
	return ENCODER_IDLE;
}

//------------------------------------------------------------------
/*void PIOINT1_IRQHandler(void){
	static volatile uint8_t i ;

	for (i=0; i < 10; i++){};

	if (encoder.pulses == 0){
		encoder.pulses += EncoderDecode();
	}

	Chip_GPIO_ClearInts(LPC_GPIO, ENCODER_PORT, (1<<ENCODER_PIN1));
}*/

//------------------------------------------------------------------
void EncoderProc(void){
	static uint8_t prevState;

	uint32_t portState = EncoderPortStateGet();

	if ((portState & (1<<ENCODER_PIN1)) > 0) {
		prevState = 1;
	} else {
		if (prevState > 0){
			encoder.pulses += EncoderDecode();
		}
		prevState = 0;
	}
}

//------------------------------------------------------------------
uint16_t EncoderActionCheck(){
	if (encoder.pulses != 0) return 1;
	return 0;
}

//------------------------------------------------------------------
int16_t EncoderValueGet(){
	int16_t buff = encoder.pulses;
	encoder.pulses = 0;
	return buff;
}
