/*
 * gen.c
 *
 *  Created on: 15 lut 2018
 *      Author: dsosnowski
 */


#include "chip.h"
#include "serialDebug.h"

#include "menu.h"
#include "settings.h"
#include "gen.h"

#define GEN_PORT					1
#define GEN_PIN						1

#define GEN_TIMER					LPC_TIMER32_1
#define GEN_PWM_CH					0
#define GEN_TIMER_PERSCALE			0
#define GEN_TICKS_PER_SECOND		48000000
#define GEN_TIMER_DEFPERIOD			48000



//-------------------------------------------------------
struct {
	uint32_t pulse;
	uint32_t period;
} gen;


//-------------------------------------------------------------------
static void TimerResetSet(LPC_TIMER_T *pTMR) {
	/* Disable timer, set terminal count to non-0 */
	pTMR->TCR = 0;
	pTMR->TC = 1;
	/* Reset timer counter */
	pTMR->TCR = TIMER_RESET;
	/* Wait for terminal count to clear */
	while (pTMR->TC != 0) {}
	pTMR->TCR = 0;
}

//-------------------------------------------------------------------
static void TimerPwmEnable(LPC_TIMER_T *pTMR, int8_t matchnum) {
	pTMR->PWMC = (1<<matchnum);
}

//-------------------------------------------------------
void GenInit(void){
	Chip_TIMER_Init(GEN_TIMER);
	TimerResetSet(GEN_TIMER);

	Chip_IOCON_PinMuxSet(LPC_IOCON, IOCON_PIO1_1, IOCON_FUNC3 | IOCON_DIGMODE_EN);
	Chip_GPIO_SetPortDIROutput(LPC_GPIO, GEN_PORT, (1<<GEN_PIN));

	//Chip_TIMER_Disable(DISPLAY_BKL_TIMER);
	Chip_TIMER_PrescaleSet(GEN_TIMER, GEN_TIMER_PERSCALE);
	Chip_TIMER_ResetOnMatchEnable(GEN_TIMER, 3);

	GenSett(SettFreqGet(),SettDutyGet());
	GenTimerConfogUpdate();

	//Chip_TIMER_SetMatch(GEN_TIMER, GEN_PWM_CH, (GEN_TIMER_DEFPERIOD / 2));
	//Chip_TIMER_SetMatch(GEN_TIMER, 3, GEN_TIMER_DEFPERIOD);
	TimerPwmEnable(GEN_TIMER, GEN_PWM_CH);
	Chip_TIMER_Enable(GEN_TIMER);
}

//-------------------------------------------------------
uint32_t GenSett(uint32_t freq, uint8_t duty){
	gen.period = (GEN_TICKS_PER_SECOND/freq);
	if (gen.period < 2) gen.period = 2;

	gen.pulse = ((gen.period * duty) / 100);

	//if (gen.period > 2) gen.period -= 1;
	//gen.pulse = (gen.period / 2);

	return GenRealFreqGet();
}

//-------------------------------------------------------
void GenTimerConfogUpdate(void){
	static volatile uint8_t i;

	Chip_TIMER_SetMatch(GEN_TIMER, 3, (gen.period - 1));
	Chip_TIMER_SetMatch(GEN_TIMER, GEN_PWM_CH, (gen.period - gen.pulse));

	GEN_TIMER->TCR = TIMER_RESET;
	for(i = 0; i < 10; i++){};
	GEN_TIMER->TCR = TIMER_ENABLE;
}

//-------------------------------------------------------
uint32_t GenRealFreqGet(void){
	uint32_t fval = (GEN_TICKS_PER_SECOND / gen.period);
	return fval;
}

//-------------------------------------------------------
uint32_t GenRealDutyGet(void){
	uint32_t dval = ((gen.pulse * 100) / gen.period);
	return dval;
}

//-------------------------------------------------------
uint32_t GenRealHimpGet(void){
	uint32_t hval = ((gen.pulse * 10) / (GEN_TICKS_PER_SECOND / 1000000));
	return hval;
}

//-------------------------------------------------------
uint32_t GenRealLimpGet(void){
	uint32_t lval = (((gen.period - gen.pulse) * 10)/ (GEN_TICKS_PER_SECOND / 1000000));
	return lval;
}

