/*
 * menu.c
 *
 *  Created on: 14.02.2018
 *      Author: Procesorowiec
 */

#include "chip.h"
#include "serialDebug.h"
#include "mydraw.h"
#include "myfonts.h"
#include "mainscreen.h"
#include "menu.h"
#include "settings.h"


const uint32_t editValues[6] = 	{1, 10, 100, 1000, 10000, 100000};
const uint32_t freqRanges[5] = 	{1000	, 20000	, 100000, 300000, 1000000};
const uint32_t freqEditMins[5] ={1		, 100	, 1000	, 10000	, 100000};
const uint8_t freqELvMins[5] = 	{0		, 2		, 3		, 4		, 5		};


#define MENU_DEFAULT_POS		MENU_FREQ_POS
#define MENU_DEFAULT_EDITLEVEL	3		//1000


struct {
	uint8_t pos;
	uint8_t freqRange;
	uint8_t editLevel;
	uint32_t editValue;
} menu;

//-------------------------------------------------------------------------
void MenuInit(void){
	menu.pos = MENU_DEFAULT_POS;
	menu.editLevel = MENU_DEFAULT_EDITLEVEL;
	menu.editValue = editValues[menu.editLevel];
}
//-------------------------------------------------------------------------
void MenuPosChange(uint8_t action){
	if (action > 0){
		menu.pos++;
		if (menu.pos >= MENU_POS_MAX){
			menu.pos = 0;
		}
	}
}
//-------------------------------------------------------------------------
uint8_t MenuPosGet(void){
	return (uint8_t)menu.pos;
}


//-------------------------------------------------------------------------
uint8_t MenuFreqRangeSet(uint32_t fval){
	uint8_t i;
	for (i = 0; i < MENU_FREQ_RANGES; i++){
		if (fval <= freqRanges[i]){
			break;
		}
	}
	menu.freqRange = i;
	return menu.freqRange;
}
//-------------------------------------------------------------------------
void MenuFreqEditLevelAutoIncr(uint8_t range){
	if (menu.editValue < freqEditMins[range]){

		menu.editLevel = freqELvMins[range];
	}
	menu.editValue = editValues[menu.editLevel];
}


//-------------------------------------------------------------------------
static uint8_t MenuFreqEditLevelCorrect(uint8_t range){
	if (menu.editValue < freqEditMins[range]){
		return 1;
	}
	return 0;
}
//-------------------------------------------------------------------------
static uint8_t MenuDutyEditLevelCorrect(uint8_t range){
	if (range >= 3) {
		return 1;
	}
	return 0;
}
//-------------------------------------------------------------------------
uint8_t MenuEditLevelChange(uint8_t action){

	MenuFreqRangeSet(SettFreqGet());

	if (menu.pos == MENU_FREQ_POS){

		if (action > 0){
			menu.editLevel++;
		}

		if (menu.editLevel >= MENU_FREQ_EDITLEVELS){
			menu.editLevel = 0;
		}

		if (MenuFreqEditLevelCorrect(menu.freqRange) > 0){
			menu.editLevel = freqELvMins[menu.freqRange];
		}

	}

	if (menu.pos == MENU_DUTY_POS){
		if (action > 0){
			menu.editLevel++;
		}

		if (menu.editLevel >= MENU_DUTY_EDITLEVELS){
			menu.editLevel = 0;
		}

		if (MenuDutyEditLevelCorrect(menu.freqRange) > 0){
			menu.editLevel = 1;
		}

	}

	menu.editValue = editValues[menu.editLevel];
	return menu.editLevel;
}

//-------------------------------------------------------------------------
uint8_t MenuEditLevelGet(void){
	return menu.editLevel;
}

//-------------------------------------------------------------------------
uint32_t MenuEditValueGet(void){
	return menu.editValue;
}



