/*
 * settings.c
 *
 *  Created on: 14.02.2018
 *      Author: Procesorowiec
 */


#include "chip.h"
#include "serialDebug.h"

#include "menu.h"
#include "settings.h"

#define SETT_DEFAULT_FREQ		1000
#define SETT_MIN_FREQ			1
#define SETT_MAX_FREQ			1000000

#define SETT_DEFAULT_DUTY		50
#define SETT_MIN_DUTY			1
#define SETT_MAX_DUTY			99


struct{
	uint32_t freq;
	uint8_t duty;
} sett;


//-------------------------------------------------------
void SettInit(void){
	SettFreqSet(SETT_DEFAULT_FREQ);
	SettDutySet(SETT_DEFAULT_DUTY);
}


//-------------------------------------------------------
void SettChange(int32_t changeVal, uint8_t menuPos){
	if (menuPos == 0){
		SettFreqChange(changeVal);
	}
	if (menuPos == 1){
		SettDutyChange(changeVal);
	}
}

//-------------------------------------------------------
void SettFreqSet(uint32_t fval){
	sett.freq = fval;
}

//-------------------------------------------------------
uint32_t SettFreqGet(void){
	return sett.freq;
}

//-------------------------------------------------------
void SettFreqChange(int32_t changeVal){
	int32_t fval = sett.freq;
	uint8_t range;

	fval += changeVal;
	if (fval < SETT_MIN_FREQ) fval = SETT_MIN_FREQ;
	if (fval > SETT_MAX_FREQ) fval = SETT_MAX_FREQ;

	range = MenuFreqRangeSet(fval);
	MenuFreqEditLevelAutoIncr(range);

	sett.freq = SettFreqPrepare(fval, range);

	//MenuFreqRangeSet(sett.freq);

}

//-------------------------------------------------------
uint32_t SettFreqPrepare(int32_t fval, uint8_t range){
	uint32_t corr = 0;
	uint32_t delta = 0;

	delta = freqEditMins[range];

	corr = fval % delta;
	fval -= corr;

	/*if (corr > (delta / 2)){
		fval += delta;
	}*/

	return fval;
}

//-------------------------------------------------------
void SettDutySet(uint32_t dval){
	sett.duty = dval;
}

//-------------------------------------------------------
uint8_t SettDutyGet(void){
	return sett.duty;
}

//-------------------------------------------------------
void SettDutyChange(int32_t changeVal){
	int32_t dval = sett.duty;
	uint8_t range;

	dval += changeVal;
	if (dval < SETT_MIN_DUTY) dval = SETT_MIN_DUTY;
	if (dval > SETT_MAX_DUTY) dval = SETT_MAX_DUTY;

	range = MenuFreqRangeSet(sett.freq);

	if (range >= 3){
		dval -= (dval % 10);
		if (dval < 10) dval = 10;
	}

	sett.duty = dval;
}
